@@ -3,8 +3,8 @@ import esphome.codegen as cg
 import esphome.config_validation as cv
 from esphome import automation
 from esphome.const import (
+    CONF_ACTIVE,
     CONF_ID,
-    ESP_PLATFORM_ESP32,
     CONF_INTERVAL,
     CONF_DURATION,
     CONF_TRIGGER_ID,
@@ -15,14 +15,17 @@ from esphome.const import (
     CONF_ON_BLE_SERVICE_DATA_ADVERTISE,
     CONF_ON_BLE_MANUFACTURER_DATA_ADVERTISE,
 )
+from esphome.core import CORE
+from esphome.components.esp32 import add_idf_sdkconfig_option
+from esphome.components import esp32_ble
 
-ESP_PLATFORMS = [ESP_PLATFORM_ESP32]
-AUTO_LOAD = ["xiaomi_ble", "ruuvi_ble"]
+DEPENDENCIES = ["esp32"]
 
 CONF_ESP32_BLE_ID = "esp32_ble_id"
 CONF_SCAN_PARAMETERS = "scan_parameters"
 CONF_WINDOW = "window"
-CONF_ACTIVE = "active"
+CONF_CONTINUOUS = "continuous"
+CONF_ON_SCAN_END = "on_scan_end"
 esp32_ble_tracker_ns = cg.esphome_ns.namespace("esp32_ble_tracker")
 ESP32BLETracker = esp32_ble_tracker_ns.class_("ESP32BLETracker", cg.Component)
 ESPBTClient = esp32_ble_tracker_ns.class_("ESPBTClient")
@@ -42,6 +45,16 @@ BLEManufacturerDataAdvertiseTrigger = esp32_ble_tracker_ns.class_(
     "BLEManufacturerDataAdvertiseTrigger",
     automation.Trigger.template(adv_data_t_const_ref),
 )
+BLEEndOfScanTrigger = esp32_ble_tracker_ns.class_(
+    "BLEEndOfScanTrigger", automation.Trigger.template()
+)
+# Actions
+ESP32BLEStartScanAction = esp32_ble_tracker_ns.class_(
+    "ESP32BLEStartScanAction", automation.Action
+)
+ESP32BLEStopScanAction = esp32_ble_tracker_ns.class_(
+    "ESP32BLEStopScanAction", automation.Action
+)
 
 
 def validate_scan_parameters(config):
@@ -51,8 +64,7 @@ def validate_scan_parameters(config):
 
     if window > interval:
         raise cv.Invalid(
-            "Scan window ({}) needs to be smaller than scan interval ({})"
-            "".format(window, interval)
+            f"Scan window ({window}) needs to be smaller than scan interval ({interval})"
         )
 
     if interval.total_milliseconds * 3 > duration.total_milliseconds:
@@ -97,9 +109,7 @@ def bt_uuid(value):
             )
         return value
     raise cv.Invalid(
-        "Service UUID must be in 16 bit '{}', 32 bit '{}', or 128 bit '{}' format".format(
-            bt_uuid16_format, bt_uuid32_format, bt_uuid128_format
-        )
+        f"Service UUID must be in 16 bit '{bt_uuid16_format}', 32 bit '{bt_uuid32_format}', or 128 bit '{bt_uuid128_format}' format"
     )
 
 
@@ -108,12 +118,20 @@ def as_hex(value):
 
 
 def as_hex_array(value):
+    value = value.replace("-", "")
+    cpp_array = [
+        f"0x{part}" for part in [value[i : i + 2] for i in range(0, len(value), 2)]
+    ]
+    return cg.RawExpression(f"(uint8_t*)(const uint8_t[16]){{{','.join(cpp_array)}}}")
+
+
+def as_reversed_hex_array(value):
     value = value.replace("-", "")
     cpp_array = [
         f"0x{part}" for part in [value[i : i + 2] for i in range(0, len(value), 2)]
     ]
     return cg.RawExpression(
-        "(uint8_t*)(const uint8_t[16]){{{}}}".format(",".join(reversed(cpp_array)))
+        f"(uint8_t*)(const uint8_t[16]){{{','.join(reversed(cpp_array))}}}"
     )
 
 
@@ -133,6 +151,7 @@ CONFIG_SCHEMA = cv.Schema(
                         CONF_WINDOW, default="30ms"
                     ): cv.positive_time_period_milliseconds,
                     cv.Optional(CONF_ACTIVE, default=True): cv.boolean,
+                    cv.Optional(CONF_CONTINUOUS, default=True): cv.boolean,
                 }
             ),
             validate_scan_parameters,
@@ -163,9 +182,14 @@ CONFIG_SCHEMA = cv.Schema(
                 cv.Required(CONF_MANUFACTURER_ID): bt_uuid,
             }
         ),
+        cv.Optional(CONF_ON_SCAN_END): automation.validate_automation(
+            {cv.GenerateID(CONF_TRIGGER_ID): cv.declare_id(BLEEndOfScanTrigger)}
+        ),
     }
 ).extend(cv.COMPONENT_SCHEMA)
 
+FINAL_VALIDATE_SCHEMA = esp32_ble.validate_variant
+
 ESP_BLE_DEVICE_SCHEMA = cv.Schema(
     {
         cv.GenerateID(CONF_ESP32_BLE_ID): cv.use_id(ESP32BLETracker),
@@ -181,6 +205,7 @@ async def to_code(config):
     cg.add(var.set_scan_interval(int(params[CONF_INTERVAL].total_milliseconds / 0.625)))
     cg.add(var.set_scan_window(int(params[CONF_WINDOW].total_milliseconds / 0.625)))
     cg.add(var.set_scan_active(params[CONF_ACTIVE]))
+    cg.add(var.set_scan_continuous(params[CONF_CONTINUOUS]))
     for conf in config.get(CONF_ON_BLE_ADVERTISE, []):
         trigger = cg.new_Pvariable(conf[CONF_TRIGGER_ID], var)
         if CONF_MAC_ADDRESS in conf:
@@ -193,7 +218,7 @@ async def to_code(config):
         elif len(conf[CONF_SERVICE_UUID]) == len(bt_uuid32_format):
             cg.add(trigger.set_service_uuid32(as_hex(conf[CONF_SERVICE_UUID])))
         elif len(conf[CONF_SERVICE_UUID]) == len(bt_uuid128_format):
-            uuid128 = as_hex_array(conf[CONF_SERVICE_UUID])
+            uuid128 = as_reversed_hex_array(conf[CONF_SERVICE_UUID])
             cg.add(trigger.set_service_uuid128(uuid128))
         if CONF_MAC_ADDRESS in conf:
             cg.add(trigger.set_address(conf[CONF_MAC_ADDRESS].as_hex))
@@ -205,11 +230,68 @@ async def to_code(config):
         elif len(conf[CONF_MANUFACTURER_ID]) == len(bt_uuid32_format):
             cg.add(trigger.set_manufacturer_uuid32(as_hex(conf[CONF_MANUFACTURER_ID])))
         elif len(conf[CONF_MANUFACTURER_ID]) == len(bt_uuid128_format):
-            uuid128 = as_hex_array(conf[CONF_MANUFACTURER_ID])
+            uuid128 = as_reversed_hex_array(conf[CONF_MANUFACTURER_ID])
             cg.add(trigger.set_manufacturer_uuid128(uuid128))
         if CONF_MAC_ADDRESS in conf:
             cg.add(trigger.set_address(conf[CONF_MAC_ADDRESS].as_hex))
         await automation.build_automation(trigger, [(adv_data_t_const_ref, "x")], conf)
+    for conf in config.get(CONF_ON_SCAN_END, []):
+        trigger = cg.new_Pvariable(conf[CONF_TRIGGER_ID], var)
+        await automation.build_automation(trigger, [], conf)
+
+    if CORE.using_esp_idf:
+        add_idf_sdkconfig_option("CONFIG_BT_ENABLED", True)
+        # https://github.com/espressif/esp-idf/issues/4101
+        # https://github.com/espressif/esp-idf/issues/2503
+        # Match arduino CONFIG_BTU_TASK_STACK_SIZE
+        # https://github.com/espressif/arduino-esp32/blob/fd72cf46ad6fc1a6de99c1d83ba8eba17d80a4ee/tools/sdk/esp32/sdkconfig#L1866
+        add_idf_sdkconfig_option("CONFIG_BTU_TASK_STACK_SIZE", 8192)
+
+    cg.add_define("USE_OTA_STATE_CALLBACK")  # To be notified when an OTA update starts
+
+
+ESP32_BLE_START_SCAN_ACTION_SCHEMA = cv.Schema(
+    {
+        cv.GenerateID(): cv.use_id(ESP32BLETracker),
+        cv.Optional(CONF_CONTINUOUS, default=False): cv.templatable(cv.boolean),
+    }
+)
+
+
+@automation.register_action(
+    "esp32_ble_tracker.start_scan",
+    ESP32BLEStartScanAction,
+    ESP32_BLE_START_SCAN_ACTION_SCHEMA,
+)
+async def esp32_ble_tracker_start_scan_action_to_code(
+    config, action_id, template_arg, args
+):
+    paren = await cg.get_variable(config[CONF_ID])
+    var = cg.new_Pvariable(action_id, template_arg, paren)
+    cg.add(var.set_continuous(config[CONF_CONTINUOUS]))
+    return var
+
+
+ESP32_BLE_STOP_SCAN_ACTION_SCHEMA = automation.maybe_simple_id(
+    cv.Schema(
+        {
+            cv.GenerateID(): cv.use_id(ESP32BLETracker),
+        }
+    )
+)
+
+
+@automation.register_action(
+    "esp32_ble_tracker.stop_scan",
+    ESP32BLEStopScanAction,
+    ESP32_BLE_STOP_SCAN_ACTION_SCHEMA,
+)
+async def esp32_ble_tracker_stop_scan_action_to_code(
+    config, action_id, template_arg, args
+):
+    var = cg.new_Pvariable(action_id, template_arg)
+    await cg.register_parented(var, config[CONF_ID])
+    return var
 
 
 async def register_ble_device(var, config):
