@@ -5,8 +5,11 @@ from esphome.const import (
     CONF_ID,
     CONF_MODEL,
     CONF_MOISTURE,
+    CONF_TEMPERATURE,
     DEVICE_CLASS_HUMIDITY,
     STATE_CLASS_MEASUREMENT,
+    UNIT_CELSIUS,
+    ICON_THERMOMETER,
 )
 
 from . import RGModel, HydreonRGxxComponent
@@ -33,13 +36,15 @@ SUPPORTED_SENSORS = {
     CONF_TOTAL_ACC: ["RG_15"],
     CONF_R_INT: ["RG_15"],
     CONF_MOISTURE: ["RG_9"],
+    CONF_TEMPERATURE: ["RG_9"],
 }
 PROTOCOL_NAMES = {
     CONF_MOISTURE: "R",
     CONF_ACC: "Acc",
-    CONF_R_INT: "Rint",
+    CONF_R_INT: "RInt",
     CONF_EVENT_ACC: "EventAcc",
     CONF_TOTAL_ACC: "TotalAcc",
+    CONF_TEMPERATURE: "t",
 }
 
 
@@ -92,6 +97,12 @@ CONFIG_SCHEMA = cv.All(
                 device_class=DEVICE_CLASS_HUMIDITY,
                 state_class=STATE_CLASS_MEASUREMENT,
             ),
+            cv.Optional(CONF_TEMPERATURE): sensor.sensor_schema(
+                unit_of_measurement=UNIT_CELSIUS,
+                accuracy_decimals=0,
+                icon=ICON_THERMOMETER,
+                state_class=STATE_CLASS_MEASUREMENT,
+            ),
         }
     )
     .extend(cv.polling_component_schema("60s"))
@@ -108,7 +119,7 @@ async def to_code(config):
     cg.add_define(
         "HYDREON_RGXX_PROTOCOL_LIST(F, sep)",
         cg.RawExpression(
-            " sep ".join([f'F("{name}")' for name in PROTOCOL_NAMES.values()])
+            " sep ".join([f'F("{name} ")' for name in PROTOCOL_NAMES.values()])
         ),
     )
     cg.add_define("HYDREON_RGXX_NUM_SENSORS", len(PROTOCOL_NAMES))
@@ -117,3 +128,5 @@ async def to_code(config):
         if conf in config:
             sens = await sensor.new_sensor(config[conf])
             cg.add(var.set_sensor(sens, i))
+
+    cg.add(var.set_request_temperature(CONF_TEMPERATURE in config))
